﻿using BarContracts.BindingModels;
using BarContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;

namespace BarView.Forms
{
	public partial class FormClients : Form
	{
		private readonly ILogger _logger;
		private readonly IClientLogic _clientLogic;
		
		public FormClients(ILogger<FormClients> Logger, IClientLogic ClientLogic)
		{
			InitializeComponent();
			
			_logger = Logger;
			_clientLogic = ClientLogic;
		}

		private void LoadData()
		{
			try
			{
				var List = _clientLogic.ReadList(null);
				
				if (List != null)
				{
					DataGridView.DataSource = List;
					DataGridView.Columns["Id"].Visible = false;
					DataGridView.Columns["ClientFIO"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
					DataGridView.Columns["Email"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
					DataGridView.Columns["Password"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
				}
				
				_logger.LogInformation("Загрузка клиентов");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки клиентов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}

		private void FormClients_Load(object sender, EventArgs e)
		{
			LoadData();
		}

		private void ButtonDelete_Click(object sender, EventArgs e)
		{
			if (DataGridView.SelectedRows.Count == 1)
			{
				if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
				{
					int Id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
					_logger.LogInformation("Удаление клиента");
					
					try
					{
						if (!_clientLogic.Delete(new ClientBindingModel
						{
							Id = Id
						}))
						{
							throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
						}
						
						LoadData();
					}
					catch (Exception ex)
					{
						_logger.LogError(ex, "Ошибка удаления компонента");
						MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
					}
				}
			}
		}
	}
}
