﻿using BarContracts.BindingModels;
using BarContracts.BusinessLogicContracts;
using BarContracts.SearchModels;
using Microsoft.Extensions.Logging;

namespace BarView.Forms
{
    public partial class FormComponent : Form
    {
        private readonly ILogger _logger;

        private readonly IComponentLogic _logic;

        private int? _id;

        public int Id { set { _id = value; } }

        public FormComponent(ILogger<FormComponent> Logger, IComponentLogic Logic)
        {
            InitializeComponent();

            _logger = Logger;
            _logic = Logic;
        }

        private void FormComponent_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                try
                {
                    _logger.LogInformation("Получение компонента");
                    var View = _logic.ReadElement(new ComponentSearchModel
                    {
                        Id = _id.Value
                    });

                    if (View != null)
                    {
                        ComponentNameTextBox.Text = View.ComponentName;
                        ComponentPriceTextBox.Text = View.Cost.ToString();
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка получения компонента");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(ComponentNameTextBox.Text))
            {
                MessageBox.Show("Заполните название", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Сохранение компонента");

            try
            {
                var model = new ComponentBindingModel
                {
                    Id = _id ?? 0,
                    ComponentName = ComponentNameTextBox.Text,
                    Cost = Convert.ToDouble(ComponentPriceTextBox.Text)
                };

                var OperationResult = _id.HasValue ? _logic.Update(model) : _logic.Create(model);

                if (!OperationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения компонента");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
