﻿using BarContracts.BindingModels;
using BarContracts.BusinessLogicsContracts;
using BarContracts.SearchModels;
using Microsoft.Extensions.Logging;

namespace BarView.Forms
{
    public partial class FormImplementer : Form
    {
        private readonly ILogger _logger;
        private readonly IImplementerLogic _logic;
        
        private int? _id;
        
        public int Id { set { _id = value; } }

        public FormImplementer(ILogger<FormImplementer> Logger, IImplementerLogic Logic)
        {
            InitializeComponent();
            
            _logger = Logger;
            _logic = Logic;
        }

        private void FormImplementer_Load(object sender, EventArgs e)
        {
            if (_id.HasValue)
            {
                try
                {
                    _logger.LogInformation("Получение исполнителя");
                    
                    var View = _logic.ReadElement(new ImplementerSearchModel
                    {
                        Id = _id.Value
                    });
                    
                    if (View != null)
                    {
                        FioTextBox.Text = View.ImplementerFIO;
                        PasswordTextBox.Text = View.Password;
                        WorkExperienceNumericUpDown.Value = View.WorkExperience;
                        QualificationNumericUpDown.Value = View.Qualification;
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка получения исполнителя");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(FioTextBox.Text))
            {
                MessageBox.Show("Заполните ФИО", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (string.IsNullOrEmpty(PasswordTextBox.Text))
            {
                MessageBox.Show("Заполните пароль", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            _logger.LogInformation("Сохранение исполнителя");
            
            try
            {
                var Model = new ImplementerBindingModel
                {
                    Id = _id ?? 0,
                    ImplementerFIO = FioTextBox.Text,
                    Password = PasswordTextBox.Text,
                    WorkExperience = (int)WorkExperienceNumericUpDown.Value,
                    Qualification = (int)QualificationNumericUpDown.Value
                };
                
                var OperationResult = _id.HasValue ? _logic.Update(Model) : _logic.Create(Model);
                if (!OperationResult)
                {
                    throw new Exception("Ошибка при создании или обновлении. Дополнительная информация в логах.");
                }
                
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения исполнителя");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
