﻿using BarContracts.BindingModels;
using BarContracts.BusinessLogicsContracts;
using BarContracts.DI;
using Microsoft.Extensions.Logging;

namespace BarView.Forms
{
    public partial class FormImplementers : Form
    {
        private readonly ILogger _logger;
        private readonly IImplementerLogic _implementerLogic;

        public FormImplementers(ILogger<FormImplementers> Logger, IImplementerLogic ImplementerLogic)
        {
            InitializeComponent();

            _logger = Logger;
            _implementerLogic = ImplementerLogic;
        }

        private void LoadData()
        {
            try
            {
                DataGridView.FillAndConfigGrid(_implementerLogic.ReadList(null));
                _logger.LogInformation("Загрузка исполнителей");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки исполнителей");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void FormImplementers_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            var Service = DependencyManager.Instance.Resolve<FormImplementer>();

            if (Service is FormImplementer Form)
            {
                if (Form.ShowDialog() == DialogResult.OK)
                {
                    LoadData();
                }
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                var service = DependencyManager.Instance.Resolve<FormImplementer>();
                if (service is FormImplementer Form)
                {
                    Form.Id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                    if (Form.ShowDialog() == DialogResult.OK)
                    {
                        LoadData();
                    }
                }
            }
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    int Id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                    _logger.LogInformation("Удаление исполнителя");

                    try
                    {
                        if (!_implementerLogic.Delete(new ImplementerBindingModel
                        {
                            Id = Id
                        }))
                        {
                            throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
                        }

                        LoadData();
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, "Ошибка удаления исполнителя");
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
