﻿using BarContracts.BindingModels;
using BarContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;

namespace BarView.Forms
{
    public partial class FormReportCocktailComponents : Form
    {
        private readonly ILogger _logger;
        private readonly IReportLogic _logic;

        public FormReportCocktailComponents(ILogger<FormReportCocktailComponents> Logger, IReportLogic Logic)
        {
            InitializeComponent();

            _logger = Logger;
            _logic = Logic;
        }

        private void FormReportCoctailComponents_Load(object sender, EventArgs e)
        {
            try
            {
                var Dict = _logic.GetCocktailComponents();

                if (Dict != null)
                {
                    DataGridView.Rows.Clear();

                    foreach (var Elem in Dict)
                    {
                        DataGridView.Rows.Add(new object[] { Elem.CocktailName, "", "" });

                        foreach (var ListElem in Elem.Components)
                            DataGridView.Rows.Add(new object[] { "", ListElem.Item1, ListElem.Item2 });

                        DataGridView.Rows.Add(new object[] { "Итого", "", Elem.TotalCount });
                        DataGridView.Rows.Add(Array.Empty<object>());
                    }
                }

                _logger.LogInformation("Загрузка списка Коктейльов по компонентам");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка Коктейльов по компонентам");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSaveToExcel_Click(object sender, EventArgs e)
        {
            using var Dialog = new SaveFileDialog { Filter = "xlsx|*.xlsx" };

            if (Dialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _logic.SaveCocktailComponentToExcelFile(new ReportBindingModel
                    {
                        FileName = Dialog.FileName
                    });

                    _logger.LogInformation("Сохранение списка Коктейльов по компонентам");
                    MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка сохранения списка Коктейльов по компонентам");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
