﻿using BarContracts.BindingModels;
using BarContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;
using Microsoft.Reporting.WinForms;

namespace BarView.Forms
{
    public partial class FormReportOrders : Form
    {
        private readonly ReportViewer _reportViewer;
        private readonly ILogger _logger;
        private readonly IReportLogic _logic;

        public FormReportOrders(ILogger<FormReportOrders> Logger, IReportLogic Logic)
        {
            InitializeComponent();
            _logger = Logger;
            _logic = Logic;
            
            _reportViewer = new ReportViewer
            {
                Dock = DockStyle.Fill
            };
            _reportViewer.LocalReport.LoadReportDefinition(new FileStream("ReportOrder.rdlc", FileMode.Open));
            
            Controls.Clear();
            Controls.Add(_reportViewer);
            Controls.Add(Panel);
        }

        private void ButtonMake_Click(object sender, EventArgs e)
        {
            if (FromDateTimePicker.Value.Date >= ToDateTimePicker.Value.Date)
            {
                MessageBox.Show("Дата начала должна быть меньше даты окончания", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            try
            {
                var DataSource = _logic.GetOrders(new ReportBindingModel
                {
                    DateFrom = FromDateTimePicker.Value,
                    DateTo = ToDateTimePicker.Value
                });
                
                var Source = new ReportDataSource("DataSetOrders", DataSource);
                
                _reportViewer.LocalReport.DataSources.Clear();
                _reportViewer.LocalReport.DataSources.Add(Source);
                
                var Parameters = new[] { 
                    new ReportParameter("ReportParameterPeriod", $"c {FromDateTimePicker.Value.ToShortDateString()} по {ToDateTimePicker.Value.ToShortDateString()}") 
                };
                _reportViewer.LocalReport.SetParameters(Parameters);

                _reportViewer.RefreshReport();
                _logger.LogInformation("Загрузка списка заказов на период {From}-{To}", FromDateTimePicker.Value.ToShortDateString(), ToDateTimePicker.Value.ToShortDateString());
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка заказов на период");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonToPdf_Click(object sender, EventArgs e)
        {
            if (FromDateTimePicker.Value.Date >= ToDateTimePicker.Value.Date)
            {
                MessageBox.Show("Дата начала должна быть меньше даты окончания", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            using var Dialog = new SaveFileDialog { Filter = "pdf|*.pdf" };
            
            if (Dialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    System.Text.Encoding.RegisterProvider(System.Text.CodePagesEncodingProvider.Instance);
                    _logic.SaveOrdersToPdfFile(new ReportBindingModel
                    {
                        FileName = Dialog.FileName,
                        DateFrom = FromDateTimePicker.Value,
                        DateTo = ToDateTimePicker.Value
                    });
                    
                    _logger.LogInformation("Сохранение списка заказов на период {From}-{To}", FromDateTimePicker.Value.ToShortDateString(), ToDateTimePicker.Value.ToShortDateString());
                    MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка сохранения списка заказов на период");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
